@@ -6,7 +6,7 @@ import typing
 from gettext import gettext as _
 
 import validators
-from gi.repository import Gdk, GLib, GObject, Gtk
+from gi.repository import GLib, GObject, Gtk
 
 from gsecrets.attachment_warning_dialog import AttachmentWarningDialog
 from gsecrets.color_widget import ColorEntryRow
@@ -207,7 +207,7 @@ class EntryPage(Gtk.Box):
         # Show more row
         for widget in self.toggeable_widget_list:
             if not widget.get_visible():
-                self.show_all_preferences_group.show()
+                self.show_all_preferences_group.props.visible = True
                 break
 
     def add_attribute_property_row(self, key, value):
@@ -240,9 +240,7 @@ class EntryPage(Gtk.Box):
         safe_entry = self.unlocked_database.current_element
         safe_entry.save_history()
 
-        self.unlocked_database.window.send_notification(
-            _("Saved in history")
-        )
+        self.unlocked_database.window.send_notification(_("Saved in history"))
 
     def _on_history_saved(self, entry):
         if not entry.history:
@@ -286,7 +284,7 @@ class EntryPage(Gtk.Box):
         for widget in self.toggeable_widget_list:
             widget.set_visible(True)
 
-        self.show_all_preferences_group.hide()
+        self.show_all_preferences_group.props.visible = False
 
     @Gtk.Template.Callback()
     def on_notes_detach_button_clicked(self, _button):
@@ -304,19 +302,26 @@ class EntryPage(Gtk.Box):
         safe_entry = self.unlocked_database.current_element
         safe_entry.props.icon = icon
 
+    def _on_launch(self, launcher, result, window):
+        try:
+            launcher.launch_finish(result)
+        except GLib.Error as err:
+            logging.debug("Could not launch uri: %s", err.message)
+            window.send_notification(_("Could not launch url"))
+
     @Gtk.Template.Callback()
     def on_visit_url_button_clicked(self, _button):
         self.unlocked_database.start_database_lock_timer()
         url = self.url_entry_row.props.text
         window = self.unlocked_database.window
-        if validators.url(url):
-            Gtk.show_uri(window, url, Gdk.CURRENT_TIME)
-        else:
+        if not validators.url(url):
             url = "https://" + url
-            if validators.url(url):
-                Gtk.show_uri(window, url, Gdk.CURRENT_TIME)
-            else:
+            if not validators.url(url):
                 window.send_notification(_("The address is not valid"))
+                return
+
+        launcher = Gtk.UriLauncher.new(url)
+        launcher.launch(window, None, self._on_launch, window)
 
     @Gtk.Template.Callback()
     def on_otp_copy_button_clicked(self, _button):
@@ -351,27 +356,24 @@ class EntryPage(Gtk.Box):
 
     def _on_add_attachment(self, _widget, _action_name, _pspec):
         self.unlocked_database.start_database_lock_timer()
-        select_dialog = Gtk.FileChooserNative.new(
-            # NOTE: Filechooser title for selecting attachment file
-            _("Select Attachment"),
+
+        dialog = Gtk.FileDialog.new()
+        # NOTE: Filechooser title for selecting attachment file
+        dialog.props.title = _("Select Attachments")
+        dialog.props.accept_label = _("_Add")
+
+        dialog.open_multiple(
             self.unlocked_database.window,
-            Gtk.FileChooserAction.OPEN,
-            _("_Add"),
             None,
+            self._on_select_filechooser_response,
         )
-        select_dialog.set_modal(True)
-        select_dialog.set_select_multiple(True)
 
-        select_dialog.connect(
-            "response", self._on_select_filechooser_response, select_dialog
-        )
-        select_dialog.show()
-
-    def _on_select_filechooser_response(
-        self, dialog: Gtk.Dialog, response: Gtk.ResponseType, _dialog: Gtk.Dialog
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
+    def _on_select_filechooser_response(self, dialog, result):
+        try:
+            files = dialog.open_multiple_finish(result)
+        except GLib.Error as err:
+            logging.debug("Could not open files: %s", err.message)
+        else:
             safe_entry: SafeEntry = self.unlocked_database.current_element
 
             def callback(gfile, result):
@@ -385,7 +387,7 @@ class EntryPage(Gtk.Box):
                     new_attachment = safe_entry.add_attachment(data, filename)
                     self.add_attachment_row(new_attachment)
 
-            for attachment in dialog.get_files():
+            for attachment in files:
                 attachment.load_bytes_async(None, callback)
 
     def add_attachment_row(self, attachment):
@@ -420,21 +422,21 @@ class EntryPage(Gtk.Box):
         if otp_token:
             remaining_time = safe_entry.otp_lifespan() / safe_entry.otp_interval()
             self.otp_progress_icon.props.progress = remaining_time
-            self.otp_token_row.show()
+            self.otp_token_row.props.visible = True
             self.otp_token_row.props.title = otp_token
             self.otp_timer_handler = GObject.timeout_add(
                 100, self.otp_update, safe_entry, None
             )
         else:
-            self.otp_token_row.hide()
+            self.otp_token_row.props.visible = False
 
         if not otp_token and safe_entry.props.otp:
             self.otp_error_revealer.reveal(True)
-            self.otp_token_row.hide()
+            self.otp_token_row.props.visible = False
             self.otp_secret_entry_row.add_css_class("error")
         else:
             self.otp_error_revealer.reveal(False)
-            self.otp_token_row.show()
+            self.otp_token_row.props.visible = True
             self.otp_secret_entry_row.remove_css_class("error")
 
         return GLib.SOURCE_CONTINUE
